#ifndef OPENTISSUE_UTILITY_GL_GL_DRAW_ARC_H
#define OPENTISSUE_UTILITY_GL_GL_DRAW_ARC_H
//
// OpenTissue Template Library
// - A generic toolbox for physics-based modeling and simulation.
// Copyright (C) 2008 Department of Computer Science, University of Copenhagen.
//
// OTTL is licensed under zlib: http://opensource.org/licenses/zlib-license.php
//
#include <OpenTissue/configuration.h>

#include <OpenTissue/utility/gl/gl.h>
#include <OpenTissue/core/math/math_functions.h>


namespace OpenTissue
{

  namespace gl
  {
    
    /**
    * Draw Arc (always bool wireframe = true)
    *
    * @param sphere     A reference to the sphere that should be drawn.
    * @param wireframe  Draw in wireframe or normal.
    */
    template<typename vector3_type>
    inline void DrawArc(vector3_type const & x0, vector3_type const & x1, typename vector3_type::value_type const & theta)
    {
      typedef typename vector3_type::value_traits  value_traits;
      typedef typename vector3_type::value_type    real_type;
      using std::fabs;

      if(fabs(theta) < value_traits::one()/4096)
      {
        glBegin(GL_LINES);
        glVertex3f(x0(0), x0(1), 0.f);
        glVertex3f(x1(0), x1(1), 0.f);
        glEnd();
        return;
      }

      size_t const segments = 16;


      using std::sin;
      using std::sqrt;
      using std::fabs;

      real_type const l = math::length(x1-x0);
      real_type const r = l/(value_traits::two()*sin(fabs(theta)));
      //real_type const r = l/(value_traits::two()*sin(theta));
      real_type const h = sqrt(r*r-l*l/value_traits::four());

      vector3_type const u = math::unit(x1-x0);
      // TODO: implement a generic 3d version
      vector3_type const u_hat = vector3_type(u(1), -u(0), value_traits::zero());
      //vector3_type const v1 = math::normalize(vector3_type(v0(1), -v0(0), value_traits::zero()));

      vector3_type const midpoint = value_traits::half()*(x1+x0);
      vector3_type const c = midpoint + math::sgn(theta)*h*u_hat;

      vector3_type p0 = x0;

      //glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
      //GLUquadricObj* qobj = gluNewQuadric();
      //glPushMatrix();
      //glTranslatef(c(0), c(1), c(2));
      //gluSphere(qobj,0.1f,32,32);
      //gluDisk(qobj,r,r,32,32);
      //glPopMatrix();
      //gluDeleteQuadric(qobj);

      //return;

      glBegin(GL_LINES);
      for(size_t n = 1; n < segments; ++n)
      {
        //<real_type const a = n*value_traits::two()*fabs(theta)/(segments-1);
        real_type const a = n*value_traits::two()*theta/(segments-1);
        vector3_type const p = math::Rz(-a)*(x0-c)+c;
        glVertex3f(p0(0), p0(1), 0.f);
        glVertex3f(p(0), p(1), 0.f);
        p0 = p;
      }
      glEnd();
    }

  } // namespace gl

} // namespace OpenTissue

//OPENTISSUE_UTILITY_GL_GL_DRAW_ARC_H
#endif
