#ifndef OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_FEATURE_SPACE_LOG_HISTOGRAM_H
#define OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_FEATURE_SPACE_LOG_HISTOGRAM_H
//
// OpenTissue Template Library
// - A generic toolbox for physics-based modeling and simulation.
// Copyright (C) 2008 Department of Computer Science, University of Copenhagen.
//
// OTTL is licensed under zlib: http://opensource.org/licenses/zlib-license.php
//
#include <OpenTissue/configuration.h>

#include <OpenTissue/core/math/math_is_finite.h>

namespace OpenTissue
{
  namespace image_analysis
  {
    /**
     * Compute Feature Space Log Histogram. 
     *
     * @param histogram      A feature space histogram.
     * @param log_histogram  Upon return contains the log histogram of the feature space histogram.
     */
    template<
        typename histogram_type  // = OpenTissue::grid::Grid< size_t>
      , typename log_histogram_type  // = OpenTissue::grid::Grid< double >
    >
    void compute_feature_space_log_histogram(
        histogram_type const & histogram
      , log_histogram_type & log_histogram
    )
    {
      using std::log;

      typedef typename histogram_type::const_index_iterator             histogram_iterator;
      typedef typename log_histogram_type::index_iterator               log_histogram_iterator;
      log_histogram.create(histogram.min_coord(),histogram.max_coord(),histogram.I(),histogram.J(),histogram.K());

      histogram_iterator hbegin = histogram.begin();
      histogram_iterator hend   = histogram.end();
      histogram_iterator h;

      log_histogram_iterator lbegin = log_histogram.begin();
      log_histogram_iterator lend   = log_histogram.end();
      log_histogram_iterator l;

      for( h=hbegin,l=lbegin; h!=hend; ++h,++l)
      {
        (*l) = log( (*h) + 1.0);
        assert( is_finite( *l )  || !"compute_feature_space_log_histogram(): NaN ");
      }
    }
  }// end namespace image_analysis

} // namespace OpenTissue

//  OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_FEATURE_SPACE_LOG_HISTOGRAM_H
#endif
