#ifndef OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_HISTOGRAM_EQUALIZATION_H
#define OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_HISTOGRAM_EQUALIZATION_H
//
// OpenTissue Template Library
// - A generic toolbox for physics-based modeling and simulation.
// Copyright (C) 2008 Department of Computer Science, University of Copenhagen.
//
// OTTL is licensed under zlib: http://opensource.org/licenses/zlib-license.php
//
#include <OpenTissue/configuration.h>

namespace OpenTissue
{
  namespace image_analysis
  {
    /**
     * Histogram Equalization.
     *
     */
    template<
        typename grid_type_in  // = OpenTissue::grid::Grid< size_t>
      , typename equalized_histogram_type  // = OpenTissue::grid::Grid< double >
    >
    void compute_histogram_equalization(
        grid_type_in const & image
      , equalized_histogram_type & equal_histogram
    )
    {
      typedef typename grid_type_in::value_type                          image_value;
      typedef typename grid_type_in::const_index_iterator                image_iterator;
      typedef typename equalized_histogram_type::index_iterator         equal_histogram_iterator;

      using std::min;
      using std::max;

      equal_histogram.create(image.min_coord(),image.max_coord(),image.I(),image.J(),image.K());

      image_value max_val = max(image);
      image_value min_val = min(image);

      image_value cnt_values = (max_val - min_val)+1;
      std::vector<size_t> n(cnt_values,0);

      image_iterator ibegin = image.begin();
      image_iterator iend   = image.end();
      image_iterator i;
      size_t N = 0;
      for(i=ibegin;i!=iend;++i)
      {
        ++n[ *i - min_val];
        ++N;
      }

      equal_histogram_iterator ebegin = equal_histogram.begin();
      equal_histogram_iterator eend   = equal_histogram.end();
      equal_histogram_iterator e;
      for(e=ebegin,i=ibegin;i!=iend;++i,++e)
      {
        size_t nk = 0;
        for(size_t k=min_val;k<=*i;++k)
          nk += n[ k - min_val];
        *e = (1.0*nk)/N;
      }
    }

  }// end namespace image_analysis

} // namespace OpenTissue

//  OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_HISTOGRAM_EQUALIZATION_H
#endif
