#ifndef OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_INTENSITY_HISTOGRAM_H
#define OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_INTENSITY_HISTOGRAM_H
//
// OpenTissue Template Library
// - A generic toolbox for physics-based modeling and simulation.
// Copyright (C) 2008 Department of Computer Science, University of Copenhagen.
//
// OTTL is licensed under zlib: http://opensource.org/licenses/zlib-license.php
//
#include <OpenTissue/configuration.h>

#include <cmath>

namespace OpenTissue
{
  namespace image_analysis
  {

    /**
    * Compute Intensity histogram.
    *
    * @param U                       The image to compute a histogram of.
    * @param histogram               Upon return holds the computed histogram. This could be declared as std::vector<size_t> histogram;
    */
    template< typename image_type,typename histogram_type >
      void compute_intensity_histogram(
        image_type const & U
      , histogram_type & histogram
      , size_t buckets = 256
      )
    {
      typedef typename image_type::const_index_iterator  intensity_iterator;
      typedef typename image_type::value_type            intensity_type;

      using std::max;
      using std::min;
      using std::floor;

      histogram.resize(buckets);
      for(size_t idx=0;idx<buckets;++idx)
        histogram[idx] = 0;

      double  min_intensity = 1.0*min(U);
      double  max_intensity = 1.0*max(U);
      double  i_width = (max_intensity-min_intensity)/buckets;

      intensity_iterator i    = U.begin();
      intensity_iterator end  = U.end();
      for(;i!=end;++i)
      {
        size_t i_bucket = static_cast<size_t>( floor((*i - min_intensity)/i_width) );

        if(i_bucket==buckets) 
          --i_bucket; 

        assert(i_bucket<buckets || !"compute_intensity_histogram(): Illegal bucket index!");

        ++(histogram[i_bucket]);
      }
    }

  }// end namespace image_analysis

} // namespace OpenTissue

//  OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_COMPUTE_INTENSITY_HISTOGRAM_H
#endif
