#ifndef OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_FEATURE_SPACE_PCA_H
#define OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_FEATURE_SPACE_PCA_H
//
// OpenTissue Template Library
// - A generic toolbox for physics-based modeling and simulation.
// Copyright (C) 2008 Department of Computer Science, University of Copenhagen.
//
// OTTL is licensed under zlib: http://opensource.org/licenses/zlib-license.php
//
#include <OpenTissue/configuration.h>

#include <OpenTissue/core/math/math_covariance.h>

namespace OpenTissue
{
  namespace image_analysis
  {
    /**
    * Feature space PCA.
    * Performs a principal component analysis on a feature space grid.
    *
    * @param feature_grid     A feature space grid (itensity, gradient magnitude, second derivative).
    * @param mean            Upon return this argument holds the mean value of the feature space grid.
    * @param C               Upon return this arguments holds the covariance of the feature space grid.
    */
    template< typename feature_grid_type, typename vector3_type, typename matrix3x3_type>
    void feature_space_pca(feature_grid_type const & feature_grid, vector3_type & mean, matrix3x3_type & C  )
    {
      using OpenTissue::math::covariance;

      covariance(feature_grid.begin(),feature_grid.end(),mean,C);
    }

  }// end namespace image_analysis

} // namespace OpenTissue

//  OPENTISSUE_UTILITY_IMAGE_ANALYSIS_IMAGE_ANALYSIS_FEATURE_SPACE_PCA_H
#endif
