#ifndef OPENTISSUE_UTILITY_VOLUME_RENDERING_VOLUME_RENDERING_TEXTURE_3D_TILE_H
#define OPENTISSUE_UTILITY_VOLUME_RENDERING_VOLUME_RENDERING_TEXTURE_3D_TILE_H
//
// OpenTissue Template Library
// - A generic toolbox for physics-based modeling and simulation.
// Copyright (C) 2008 Department of Computer Science, University of Copenhagen.
//
// OTTL is licensed under zlib: http://opensource.org/licenses/zlib-license.php
//
#include <OpenTissue/configuration.h>

#include <OpenTissue/gpu/texture/texture_texture3D.h>
#include <OpenTissue/core/geometry/geometry_aabb.h>
#include <cassert>

namespace OpenTissue
{
  namespace volume_rendering
  {

    /**
    * Local Data Container, used to keep information about texture tiling.
    */
    template<typename math_types>
    class Texture3DTile
    {
    public:

      typedef          Texture3DTile<math_types>  tile_type;
      typedef typename math_types::real_type      real_type;
      typedef typename math_types::vector3_type   vector3_type;
      typedef          geometry::AABB<math_types> aabb_type;

    public:

      real_type m_aabb_min_x;  ///< Parameters for AABB containing all the voxel data of the tile that should be visualized (no border voxels).
      real_type m_aabb_min_y;
      real_type m_aabb_min_z;    
      real_type m_aabb_width;
      real_type m_aabb_height;
      real_type m_aabb_depth;
      aabb_type m_aabb;        ///< Just another way to store the same AABB as above.

      real_type m_tile_min_x;  ///< AABB enclosing all the voxels of the tile, including the border voxels. These parameters are used for transforming model space points into  texture coordinates 
      real_type m_tile_min_y;
      real_type m_tile_min_z;    
      real_type m_tile_width;
      real_type m_tile_height;
      real_type m_tile_depth;    

      int       m_image_skip_i;            ///< Parameters indicating the starting position inside the voxel data, which corresponds to the first voxel in the texture tile.
      int       m_image_skip_j;
      int       m_image_skip_k;    

      int       m_image_size_i;            ///< The image size.
      int       m_image_size_j;
      int       m_image_size_k;    

      int       m_texture_size_i;          ///< The size of the texture tile, including border voxels.
      int       m_texture_size_j;
      int       m_texture_size_k;    

      int       m_texture_fill_i;          ///< The amount of voxel data in the tile, that is occupied, border pixels are ignored.
      int       m_texture_fill_j;
      int       m_texture_fill_k;    

      vector3_type      m_vertices[8];     ///< Model space vertices of AABB enclosing voxels of texture tile that should be visualized.
      vector3_type      m_eye_vertices[8]; ///< Eye space vertices of AABB enclosing voxels of texture tile that should be visualized.
      OpenTissue::texture::texture3D_pointer m_volume_texture;  ///< The volume texture containing voxel data.
      real_type         m_min_depth;       ///< Minimum depth of enclosing AABB.
      real_type         m_max_depth;       ///< Minimum depth of enclosing AABB.
      real_type         m_z_depth;         ///< For depth sorting.

    public:

      bool operator<(tile_type const & tile)const{   return (m_z_depth<tile.m_z_depth)?true:false; } //--- for depth sorting
      bool operator>(tile_type const & tile)const{   return (m_z_depth>tile.m_z_depth)?true:false; } //--- for depth sorting

    };

  } // namespace volume_rendering
} // namespace OpenTissue

// OPENTISSUE_UTILITY_VOLUME_RENDERING_VOLUME_RENDERING_TEXTURE_3D_TILE_H
#endif
