// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#include "auxil.h"
#include <cmath>

#include <iostream>

// Color maps
vector3 jet (const double x)
{
  const double r = (x >= 3.0/8.0 && x < 5.0/8.0) * (4.0 * x - 3.0/2.0)
                 + (x >= 5.0/8.0 && x < 7.0/8.0) 
                 + (x >= 7.0/8.0) * (-4.0 * x + 9.0/2.0);
  const double g = (x >= 1.0/8.0 && x < 3.0/8.0) * (4.0 * x - 1.0/2.0)
                 + (x >= 3.0/8.0 && x < 5.0/8.0) 
                 + (x >= 5.0/8.0 && x < 7.0/8.0) * (-4.0 * x + 7.0/2.0);
  const double b = (x < 1.0/8.0) * (4.0 * x + 1.0/2.0)
                 + (x >= 1.0/8.0 && x < 3.0/8.0)
                 + (x >= 3.0/8.0 && x < 5.0/8.0) * (-4.0 * x + 5.0/2.0);

  return vector3 (r, g, b);
}

vector3 hot (const double x)
{
  const double r = (x < 2.0/5.0) * (5.0/2.0 * x) + (x >= 2.0/5.0);
  const double g = (x >= 2.0/5.0 && x < 4.0/5.0) * (5.0/2.0 * x - 1.0) + (x >= 4.0/5.0);
  const double b = (x >= 4.0/5.0) * (5.0*x - 4.0);

  return vector3 (r, g, b);
}

double bessi0 (double x)
{
  const double abs_x = fabs (x);
  double retval;

  if (abs_x < 3.75)
    {
      const double y = square (x / 3.75);
      retval = 1.0 + y * (3.5156229 + y * (3.0899424 + y * (1.2067492 + 
                     y * (0.2659732 + y * (0.360768e-1 + y * 0.45813e-2)))));
    }
  else
    {
      const double y = 3.75 / abs_x;
      retval = (exp (abs_x) / sqrt(abs_x)) * (0.39894228 + y * (0.1328592e-1 + 
               y * (0.225319e-2 + y * (-0.157565e-2 + y * (0.916281e-2 +
               y * (-0.2057706e-1 + y * (0.2635537e-1 + y * (-0.1647633e-1 +
               y * 0.392377e-2))))))));
    }

  return retval;
}

