// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#ifndef BONE_APPEARANCE_H
#define BONE_APPEARANCE_H

#include <OpenTissue/kinematics/skeleton/skeleton_types.h>

// Bunch of typedefs because OpenTissue _really_ likes namespaces (XXX: move these to a separate file (typedefs.h) ?)
typedef OpenTissue::math::default_math_types math_types;
typedef OpenTissue::skeleton::DefaultBoneTraits <math_types> default_bone_traits;

/**
 Simple class for handling the visual apperance of a bone (i.e. it's width).
 */
class bone_appearance : public default_bone_traits
{
public:
  // Constructor
  bone_appearance ();
  
  // Accessor
  /**
   Set the size of the bone.
   
   @param [in] _width
   The width of the bone.
   
   @param [in] _length
   The length of the bone.
   */
  void set_size (const double _width, const double _length);
  
  /**
   @return
   The width of the bone.
   */
  double get_width () const;
  
  /**
   @return
   The length of the bone.
   */
  double get_length () const;
  
  /**
   @return
   The square length of the bone.
   */
  double get_length2 () const;
  
private:
  // Data
  double width, length2;
};
#endif // BONE_APPEARANCE_H
