// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#include "glauxil.h"
#include <cmath>

void partial_cylinder (float radius, float height, float theta_start, float theta_stop)
{
  const float num_theta = 10;
  const float delta_theta = (theta_stop - theta_start) / num_theta;
  
  glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
  
  glBegin (GL_QUAD_STRIP);
    for (float t = theta_start - delta_theta; t <= theta_stop; t += delta_theta)
     {
      glNormal3f (cos (t), 0.0, sin (t));

//      glVertex3f (radius * cos (t), 0.0, radius * sin (t));
//      glVertex3f (radius * cos (t), height, radius * sin (t));
      glVertex3f (radius * cos (t), radius * sin (t), 0.0);
      glVertex3f (radius * cos (t), radius * sin (t), height);
    }
  glEnd ();
}

void draw_camera (const vector3 &cam)
{
  double x_signs [] = {+1, +1, -1, -1, +1};
  double y_signs [] = {+1, -1, -1, +1, +1};

  const vector3 v1 (0, 0, 1);
  const vector3 v2 (1.0 / 5.0, 2.0 / 5.0, 0);
  
  const double radius = 0.5;
  const vector3 a = cam + radius * v1;
  const vector3 b = cam - radius * v1;

  glBegin (GL_QUADS);
  for (size_t t = 0; t < 4; t ++)
    {
      size_t idx = t;
      const vector3 a1 (a (0) + x_signs [idx] * v2 (0), a (1) + y_signs [idx] * v2 (1), a (2));
      const vector3 b1 (b (0) + x_signs [idx] * v2 (0), b (1) + y_signs [idx] * v2 (1), b (2));
      glVertex3f (a1 (0), a1 (1), a1 (2));
      glVertex3f (b1 (0), b1 (1), b1 (2));

      idx ++;
      const vector3 b2 (b (0) + x_signs [idx] * v2 (0), b (1) + y_signs [idx] * v2 (1), b (2));
      const vector3 a2 (a (0) + x_signs [idx] * v2 (0), a (1) + y_signs [idx] * v2 (1), a (2));
      glVertex3f (b2 (0), b2 (1), b2 (2));
      glVertex3f (a2 (0), a2 (1), a2 (2));
    }
  glEnd ();

  glBegin (GL_QUADS);
  for (size_t t = 0; t < 4; t ++)
    {
      const vector3 a1 (a (0) + x_signs [t] * v2 (0), a (1) + y_signs [t] * v2 (1), a (2));
      glVertex3f (a1 (0), a1 (1), a1 (2));
    }
  glEnd ();

  glBegin (GL_QUADS);
  for (size_t t = 0; t < 4; t ++)
    {
      const vector3 b1 (b (0) + x_signs [t] * v2 (0), b (1) + y_signs [t] * v2 (1), b (2));
      glVertex3f (b1 (0), b1 (1), b1 (2));
    }
  glEnd ();

  GLUquadric *qobj = gluNewQuadric ();
  glPushMatrix();
  const vector3 trans = cam - 0.75 * v1;
  glTranslatef (trans (0), trans (1) , trans (2));
  gluCylinder (qobj, radius/4.0, radius/4.0, 2.0*radius, 12, 1);
  glPopMatrix ();
}
