// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#ifndef GUI_H
#define GUI_H

#include <OpenTissue/configuration.h>

#define DEFINE_GLUT_MAIN
#include <OpenTissue/utility/glut/glut_perspective_view_application.h>
#undef DEFINE_GLUT_MAIN

#include "pf_skeleton_tracker.h"
#include "auxil.h"
#include "options.h"
#include "glauxil.h"
#include "opencv_helper.h"

/**
 A basic class for 3D visualisations of different trackers.
 */
template <class tracker_type>
class TrackerGUI : public OpenTissue::glut::PerspectiveViewApplication
{
protected:
  const std::string title;
  calibration calib;
  tracker_type track;
  const bool playback;

public:
  TrackerGUI (const std::string _title, const options &opts, const bool _playback)
    : title (_title),
      calib (opts),
      track (opts, calib, _playback),
      playback (_playback)
    {
      this->z_far () = 5000; // what does this do?
    }
  
  const char* do_get_title () const
    {
      return title.c_str ();
    }

  void do_display ()
    {
      static bool in_progress = false;
      if (in_progress) return;
      
      in_progress = true;
      OpenTissue::gl::ColorPicker (0.5, 0.5, 0.5);
      //draw_camera (cam);
      track.show ();
      in_progress = false;
    }

  void do_action (unsigned char choice)
    {
      std::cout << "You pressed " << choice << std::endl;
      switch (choice)
        {
          case 't':
            run ();
            break;
        };
    }

  void do_init_right_click_menu (int main_menu, void menu (int entry))
    {
    }

  void do_init ()
    {
      this->camera ().move (80);
      glClearColor (0.9, 0.9, 0.9, 0.0);
      std::cout << "Press 't' to get going" << std::endl;
    }

  void do_run ()
    {
      const bool valid = track.refresh ();
      track.prepare_mesh ();
      if (valid)
        {
          display ();
          action ('y');
          if (!playback)
            track.save_states ();
        }
      else
        action (' ');
    }

  void do_shutdown ()
    {
    }
};

#endif // GUI_H
