/**
 * This file is part of the nestk library.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Nicolas Burrus <nicolas.burrus@uc3m.es>, (C) 2010
 */

#include "mesh.h"

#include <ntk/utils/opencv_utils.h>

#include <opencv/cv.h>

#include <cstring>

using namespace cv;

namespace ntk
{

  void Mesh::applyTransform(const Pose3D& pose)
  {
    foreach_idx(i, vertices)
      vertices[i] = pose.cameraTransform(vertices[i]);
  }

  Point3f Mesh :: centerize()
  {
    Point3f center(0,0,0);
    foreach_idx(i, vertices)
    {
      center += vertices[i];
    }
    center *= 1.0/vertices.size();

    foreach_idx(i, vertices)
    {
      vertices[i] -= center;
    }
    return center;
  }

  void Mesh:: clear()
  {
    vertices.clear();
    colors.clear();
    normals.clear();
    texcoords.clear();
    faces.clear();
    texture = cv::Mat3b();
  }

} // end of ntk
