/**
 * This file is part of the nestk library.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Nicolas Burrus <nicolas.burrus@uc3m.es>, (C) 2010
 */

#ifndef NTK_MESH_MESH_H
# define NTK_MESH_MESH_H

#include <ntk/core.h>
#include <ntk/camera/calibration.h>
#include <ntk/geometry/pose_3d.h>

#include <vector>

namespace ntk
{

  class Face
  {
  public:
    Face(unsigned i1, unsigned i2, unsigned i3)
    { indices[0] = i1; indices[1] = i2; indices[2] = i2; }

    Face() {}

    unsigned indices[3];
    static int numVertices() { return 3; };
  };

  class Mesh
  {
  public:
    std::vector<cv::Point3f> vertices;
    std::vector<cv::Vec3b> colors;
    std::vector<cv::Point3f> normals;
    std::vector<cv::Point2f> texcoords;
    std::vector<Face> faces;
    cv::Mat3b texture;

  public:
    cv::Point3f centerize();
    void applyTransform(const Pose3D& pose);

  public:
    bool hasColors() const { return colors.size() > 0; }
    bool hasNormals() const { return normals.size() > 0; }
    bool hasTexcoords() const { return texcoords.size() > 0; }
    bool hasFaces() const { return faces.size() > 0; }

  public:
    void clear();
  };
} // ntk

#endif // NTK_MESH_MESH_H
