// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#ifndef OBSERVATION_H
#define OBSERVATION_H

/**
 An abstract class providing the interface for observational sequential data.
 */
template<typename observation_type>
class observation
{
  public:
    virtual ~observation () {}; // Just here to remove a warning
    /**
     Read the data for the next frame.
     
     @param[in] goto_next
       If true read the data for the next frame. If false, re-read the current
       frame.
     
     @return
       If a new frame could be read, return true. Otherwise return false.
     */
    virtual bool refresh (bool goto_next = true) = 0;

    /**
     Visualise the data.
     */
    virtual void show () const = 0;
};
#endif // OBSERVATION_H
