// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#ifndef SKELETON_POSE_ESTIMATOR_H
#define SKELETON_POSE_ESTIMATOR_H

#include "skeleton_state.h"
#include "angle_state.h"
#include <vector>
#include "auxil.h"
#include <omp.h>

template <class state_type>
void mean (const std::vector<double> &weights, std::vector<state_type*> states,
           state_type &result)
{
  solver_type::vector_type &ret_theta = result.get_theta ();
  const int theta_size = states [0]->theta_size ();
  ret_theta.resize (theta_size);
  
  const int num_states = states.size ();
  for (int k = 0; k < theta_size; k++)
    {
      ret_theta [k] = 0;
      for (int p = 0; p < num_states; p++)
        ret_theta [k] += weights [p] * states [p]->get_theta (k);
    }

  vector3 root (0, 0, 0);
  for (int p = 0; p < num_states; p++)
    root += weights [p] * states [p]->get_root ();
  result.set_root (root);
}

template <class state_type>
void max (const std::vector<double> &weights, std::vector<state_type*> states,
          state_type &result)
{
  const int idx = find_max_idx (weights);
  result = *states [idx];
}

#endif // SKELETON_POSE_ESTIMATOR_H
