// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.

/** \mainpage HUMIM Tracker C++ API
 The HUMIM project provides a C++ framework for developing Bayesian trackers
 for human pose estimation. The code is designed to be flexible in the sense
 that it is easy to replace likelihoods and priors. An overview of the design
 is given in the technical report <a href="techreport.pdf">[ PDF ]</a>.
 
 To compile the software you will need to following support libraries
 <ul>
   <li>OpenCV</li>
   <li>ANN</li>
   <li>Boost</li>
   <li>Boost Bindings</li>
   <li>QHull</li>
   <li>Atlas</li>
   <li>DeVIL</li>
   <li>GLEW</li>
   <li>GLUT</li>
 </ul>
 With these the software should compile out of the box using CMake.
 
 When compiled, you will have the following executables
 <ul>
   <li><b>gui_tracker</b> -- A tracker that visualises the results using in 3D OpenGL.</li>
   <li><b>gui_projecter</b> -- A tracker that visualises the results in the image plane.</li>
   <li><b>headless_tracker</b> -- A tracker that does not visualise results at all. This can be useful on clusters etc.</li>
   <li><b>poser</b> -- A program that allows you to change the pose of a skeleton to fit the data.</li>
   <li><b>skeleton_sizer</b> -- A program that allows you to change the size of limbs in a skeleton.</li>
   <li><b>view_data</b> -- A program for viewing 3D point clouds.</li>
 </ul>
 
 To run a program you need a configuration file that sets parameters, data paths,
 output paths, etc. The name of this file should be given as first input to all
 programs. The tracking programs also support an optional second argument --playback
 that replays previous tracking data rather than perform the tracking again.
 An example configuration file can be seen here
 @code
observation_type          point_cloud
state_type                projection_state
data_template             Movies3D/triangles/hands2_060410/image_%.3d.points
left_image_template       Movies3D/triangles/hands2_060410/image_2_%.3d.png
background_image          Movies3D/triangles/hands2_060410/Background_2.png
intrinsic_matrix          838.08544922 838.90100098 496.17398071 392.37945557
data_start_idx            1
output_directory          output
num_particles             150
initial_pose              data/hands2_060410_1.pose
pred_noise                0.15
external_view             data/hands2_060410.view
bone_appearance           data/soren_thin_appearance_milimeters.txt
skeleton_xsf              data/soren_milimeters.xsf
measure_threshold         2
measure_variance          0.1
measure_downsampling      10
mocap_filename            Movies3D/triangles/hands2_060410/hands2_060410.mocap 
 @endcode
 
 In general you should not expect this code to be something you just run and get
 a result. It is designed for development of new tracking techniques and not for
 developing applications of tracking. So some hacking is to be expected to get
 you what you want.
 */
 
double convex_lambda = 0;

