// Copyright (C) 2011 Soren Hauberg
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 3
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
 
#include <OpenTissue/configuration.h>

#define DEFINE_GLUT_MAIN
#include <OpenTissue/utility/glut/glut_perspective_view_application.h>
#undef DEFINE_GLUT_MAIN

#include "point_cloud.h"
#include "kinect_cloud.h"
#include "triclops_cloud.h"
#include "pf_skeleton_tracker.h"
#include "auxil.h"
#include "options.h"
#include "macros.h"
#include "mocap_data.h"

/**
 A simple application for viewing point clouds.
 */
template <class cloud_type>
class Application : public OpenTissue::glut::PerspectiveViewApplication
{
protected:
  const calibration calib;
  cloud_type cloud;
  mocap_data<cloud_type> mocap;
  int frame;
  
public:
  Application (const options &opts)
    : calib (opts),
      cloud (opts, calib),
      mocap (opts, calib),
      frame (opts.data_start_idx ())
    {
      this->z_far () = 5000; // what does this do?
    }
  
  const char* do_get_title () const
    {
      return "Data Viewer";
    }

  void do_display ()
    {
      static bool in_progress = false;
      if (in_progress) return;
      
      in_progress = true;
      glPushMatrix ();
      glScalef (0.5, 0.5, 0.5);
      glTranslatef (0, -3, 0);
      cloud.show ();
      mocap.show ();
      glPopMatrix ();
      in_progress = false;
    }

  void do_action (unsigned char choice)
    {
      switch (choice)
        {
          case 't':
            run ();
            break;
          default:
            std::cout << "You pressed " << choice << std::endl;
            break;
        };
    }

  void do_init_right_click_menu (int main_menu, void menu (int entry))
    {
    }

  void do_init ()
    {
      this->camera ().move (80);
      glClearColor (0.9, 0.9, 0.9, 0.0);
      std::cout << "Press 't' to get going" << std::endl;
    }

  void do_run ()
    {
      static bool running = true;
      if (running)
        {
          running = cloud.refresh ();
          mocap.refresh ();
          //action ('y');
          std::cout << "Frame number " << frame++ << std::endl;
        }
    }

  void do_shutdown ()
    {
    }
};

#define EXPAND(STATE_NAME, STATE_TYPE, OBS_NAME, OBS_TYPE)                     \
        if (st == STATE_NAME && ot == OBS_NAME)                                \
          {                                                                    \
            instance.reset (new Application< OBS_TYPE > (opts));               \
          }

OpenTissue::glut::instance_pointer init_glut_application (int argc, char *argv [])
{
  // Disable localisation as that screws with punctuation in numerical IO
  setenv ("LANG", "", 1);

  // Get options
  const std::string options_filename = (argc > 1) ? argv [1] : "config/config_hands2_060410.txt";
  const options opts (options_filename);

  // Create GUI
  const tracker_state_type st = opts.state_type ();
  const tracker_observation_type ot = opts.observation_type ();

  OpenTissue::glut::instance_pointer instance;
  
  EXPAND_ALL_TRACKERS
  
  return instance;
}

